from setuptools import setup

setup(name='vrconcept_arhdron',
      version='0.5',
      description='VR Concept Arhipelag dron',
      packages=['vrconcept_arhdron'],
      author_email='govoruxin.a1999@gmail.com',
      zip_safe=False)
